@@ -29,6 +29,8 @@
 #include <stdlib.h>
 #include <sys/wait.h>
 #include <string.h>
+#include <stdio.h>
+#include <stdbool.h>
 
 #include "cmd_flags.h"
 #include "debug.h"
@@ -36,9 +38,9 @@
 /* Time to wait for a process to exit */
 #define KILL_TIMEOUT_USEC 100000
 
-static int sig_rd_fd;
-static int sig_wr_fd;
-static int chld_rd_fd; /* fd to read stdout of child */
+static int sig_rd_fd = -1;
+static int sig_wr_fd = -1;
+static int chld_rd_fd = -1; /* fd to read stdout of child */
 
 /**
  * Restore the SIG CHLD handler to default behaviour
@@ -127,6 +129,7 @@ void handle_SIGCHLD_event(void)
 {
      int sig;
      read(sig_rd_fd, &sig, sizeof(int));
+     D("Seen SIGCHLD event\n");
 }
 
 /**
@@ -144,26 +147,53 @@ int get_chld_out_fd(void)
  * Handle the event of the child writing to stdout / stderr
  *
  * @param[in] fd
+ *
+ * @return false if failed
  */
-void handle_chld_out_event(int fd)
+bool handle_chld_out_event(int fd)
 {
-     static char linebuf[128];
+     static char linebuf[256];
      static int pos = 0;
 
      int n;
-     if((n = read(fd, &linebuf, sizeof(linebuf)-pos)) > 0)
+     int left = sizeof(linebuf) - pos - 1;
+     
+     if(left <= 0)
      {
-          char * pLF;
+          D("chld stdout > 256 without \n");
+          D("[CHLD]:%s\n", linebuf);
+          pos = 0;
+          left = sizeof(linebuf) -1;
+     }
+
+     if((n = read(fd, &linebuf[pos], left)) > 0)
+     {
+          char * end;
 
           pos += n;
           linebuf[pos] = '\0';
-          while( (pLF = strchr(linebuf, '\n')) != NULL)
+          while( (end = strchr(linebuf, '\n')) != NULL)
           {
-               *pLF = '\0';
-               D("[CHLD]:%s\n", linebuf);
-               strcpy(linebuf, &pLF[1]);
-               pos -= (pLF + 1 - linebuf);
+               char * p = end;
+               while((p >= linebuf) && ((*p == '\r') || (*p == '\n') || (*p == ' ') || (*p == '\t')))
+               {
+                    p--;
+               }
+               p[1] = '\0';
+               if(p > linebuf)
+               {
+                    D("[CHLD]:%s\n", linebuf);
+               }
+               pos -= (&end[1] - linebuf);
+               memmove(linebuf, &end[1], pos+1);
           }
+          return true;
+     }
+     else
+     {
+          D("[CHLD]:%s\n", linebuf);
+          pos = 0;
+          return false;
      }
 }
 
@@ -255,6 +285,7 @@ pid_t spawn_app(char * command, const int flags)
           {
               close(i);
           }
+//          fprintf(stderr, "Started\n");
 
           execvp(app_argv[0], app_argv);
           D("Execvp failed. (errno=%d)\n", errno);
@@ -275,37 +306,59 @@ pid_t spawn_app(char * command, const int flags)
  *
  * @param[in] pid The child pid
  *
- * @return (-2) If app crashed, (-1) If app exited OK, (0) If child dettached,
- * (1) If still running
+ * @return (-2) If app crashed, (-1) If app exited but with error, 
+ * (0) If child dettached or exited successfully, (1) If still running
  */
 int wait_child(pid_t pid)
 {
-    int status;
+     int retVal = 1;
+     int status;
 
      if(waitpid(pid, &status, WNOHANG) != 0)
      {
-          if(chld_rd_fd >= 0)
+          while(chld_rd_fd >= 0)
           {
-               close(chld_rd_fd);
+               fd_set fds;
+               struct timeval timeout;
+               timeout.tv_usec = 100000;
+               timeout.tv_sec = 0;
+        
+               FD_ZERO(&fds);
+               FD_SET(chld_rd_fd, &fds);
+               if(select(chld_rd_fd + 1, &fds, NULL, NULL, &timeout) > 0)
+               {
+                    if(!handle_chld_out_event(chld_rd_fd))
+                    {
+                         close(chld_rd_fd);
+                         chld_rd_fd = -1;
+                    }
+               }
+               else
+               {
+                    close(chld_rd_fd);
+                    chld_rd_fd = -1;
+               }
           }
 
-          /* If Application completed is a bad way, then lets give up now */
+          /* If Application completed in a bad way, then lets give up now */
           if (!WIFEXITED(status))
           {
                D("Process dumped core or something...\n");
-               return -2;
+               retVal = -2;
           }
-
-          if (WEXITSTATUS(status))
+          else if (WEXITSTATUS(status))
           {
                D("Process exited with status: %d\n", WEXITSTATUS(status));
-               return -1;
+               retVal = -1;
+          }
+          else
+          {
+               D("Child has stopped or detached\n");
+               retVal = 0;
           }
-
-          D("Child has stopped or detached\n");
-          return 0;
      }
-     return 1;
+//     D("wait_child retVal=%i\n", retVal);
+     return retVal;
 }
 
 
