@@ -1338,7 +1338,7 @@ static void terminate(SwallowMutex_t * mutex, pid_t pid, Display * dpy)
  *
  * @return none
  */
-static void check_pipe_fd_events(Display * dpy, SwallowMutex_t * mutex, pid_t pid)
+static void check_pipe_fd_events(Display * dpy, SwallowMutex_t * mutex)
 {
      struct PipeMsg_s msg;
      int n;
@@ -1349,7 +1349,7 @@ static void check_pipe_fd_events(Display * dpy, SwallowMutex_t * mutex, pid_t pi
      if((n == 0) || ((n < 0) && (errno != EINTR)))
      {
           D("Pipe read error, returned n=%i\n", n);
-          terminate(mutex, pid, dpy);
+          terminate(mutex, sig_globals.childPid, dpy);
 	  exit(EX_UNAVAILABLE);
      }
 
@@ -1374,7 +1374,7 @@ static void check_pipe_fd_events(Display * dpy, SwallowMutex_t * mutex, pid_t pi
 
           case SHUTDOWN_MSG:
                D("Read Pipe, got SHUTDOWN_MSG, terminating..\n");
-               terminate(mutex, pid, dpy);
+               terminate(mutex, sig_globals.childPid, dpy);
                exit(EXIT_SUCCESS);
 
           default:
@@ -1421,37 +1421,44 @@ static void check_pipe_fd_events(Display * dpy, SwallowMutex_t * mutex, pid_t pi
 /**
  * Check the app status
  *
- * @param[in] pid The process ID of the mutex
  * @param[in] flags The command flags
+ *
+ * @return -2 if broken, -1 
  */
-static int check_app_status(pid_t pid, unsigned flags)
+static int check_app_status(unsigned flags)
 {
-     int status = wait_child(pid);
+     int retVal = 1;
+     int status = wait_child(sig_globals.childPid);
+             
      switch(status)
      {
-     case -2:
+     case -2:  /* App crashed */
+          sig_globals.childPid = -1;
           exit(EX_UNAVAILABLE);
           break;
 
-     case -1:
+     case -1: /* App exited OK */
+          sig_globals.childPid = -1;
           if (!(flags & H_IGNORE_ERRORS))
           {
                exit(WEXITSTATUS(status));
           }
           break;
 
-     case 0:
+     case 0: /* App deattached */
+          sig_globals.childPid = -1;
           if((flags & H_DAEMON) == 0)
           {
-               return -1;
+               retVal = -1;
           }
           else
           {
                D("Child process has detached, keep going\n");
-               return 0;
+               retVal = 0;
           }
+     /* case 1:  App still running */
      }
-     return 1;
+     return retVal;
 }
 
 /**
@@ -1459,18 +1466,17 @@ static int check_app_status(pid_t pid, unsigned flags)
  * Use select to check if any events need processing
  *
  * @param[in] dpy The display pointer
- * @param[in] pid The child PID
  * @param[in] mutex The swallow mutex
  *
  * @return none
  */
-static void check_all_events(Display * dpy, pid_t pid, SwallowMutex_t * mutex)
+static void check_all_events(Display * dpy, SwallowMutex_t * mutex)
 {
      int sig_chld_fd = get_SIGCHLD_fd();
-     int rd_chld_fd = get_chld_out_fd();
 
      while(1)
      {
+          int rd_chld_fd = get_chld_out_fd();
 	  int selectRetVal;
           int maxfd = 0;
 
@@ -1513,13 +1519,14 @@ static void check_all_events(Display * dpy, pid_t pid, SwallowMutex_t * mutex)
           }
 
           D("SELECT IN %s\n", pTimeout ? "with timeout" : "");
-
           selectRetVal = select(maxfd + 1, &fds, NULL, NULL, pTimeout);
+          D("SELECT OUT\n");
+
           if(selectRetVal > 0)
           {
 	       if (FD_ISSET(pipe_fd, &fds))
                {
-	            check_pipe_fd_events(dpy, mutex, pid);
+	            check_pipe_fd_events(dpy, mutex);
                }
                if( FD_ISSET(sig_chld_fd, &fds))
                {
@@ -1539,11 +1546,10 @@ static void check_all_events(Display * dpy, pid_t pid, SwallowMutex_t * mutex)
           {
                D("Select exited unexpected, errno = %i\n", errno);
           }
-          D("SELECT OUT\n");
 
-          if(pid >= 0)
+          if(sig_globals.childPid >= 0)
           {
-               int retVal = check_app_status(pid, flags);
+               int retVal = check_app_status(flags);
 	       if(retVal < 0)
                {
                     return;
@@ -1551,7 +1557,6 @@ static void check_all_events(Display * dpy, pid_t pid, SwallowMutex_t * mutex)
                if(retVal == 0)
                {
                    sig_globals.childPid = -1;
-                   pid = -1;
                }
           }
      }
@@ -1562,19 +1567,16 @@ static void check_all_events(Display * dpy, pid_t pid, SwallowMutex_t * mutex)
  * Wait for application to finish and exit helper if a problem
  *
  * @param[in] dpy The display pointer
- * @param[in] pid process ID of the application
  * @param[in] mutex The swallow mutex
  *
  * @return none
  */
-static void handle_app(Display * dpy, pid_t pid, SwallowMutex_t * mutex)
+static void handle_app(Display * dpy, SwallowMutex_t * mutex)
 {
-     sig_globals.childPid = pid;
-
      if (flags & H_SWALLOW)
      {
           /* Whilst waiting for the Application to complete, check X events */
-	  check_all_events(dpy, pid, mutex);
+	  check_all_events(dpy, mutex);
 
 #ifdef USE_MUTEX_LOCK
           /* Make sure the semaphore has been released */
@@ -1584,12 +1586,13 @@ static void handle_app(Display * dpy, pid_t pid, SwallowMutex_t * mutex)
      else if(flags & H_DAEMON)
      {
           /* If Daemon, then it is not supposed to exit, so we exit instead */
+          D("App has become a daemon, so exit\n");
           terminate(mutex, -1, dpy);
           exit(EXIT_SUCCESS);
      }
      else
      {
-	  check_all_events(NULL, pid, NULL);
+	  check_all_events(NULL, NULL);
      }
 
      sig_globals.childPid = -1;
@@ -1783,7 +1786,6 @@ int main(int argc, char **argv)
      {
 	  int loops = 1;
 	  pid_t pid;
-          int maxFd;
 
 	  /* This application will use the $repeat variable */
 	  if (flags & H_REPEATCOUNT)
@@ -1823,37 +1825,26 @@ int main(int argc, char **argv)
 	       XSync(dpy, False);
 	  }
 
-          maxFd = pipe_fd;
-          if(dpy && (ConnectionNumber(dpy) > maxFd))
-          {
-               maxFd = ConnectionNumber(dpy);
-          }
-
           pid = spawn_app(command, flags);
 	  if(pid == -1)
           {
-#ifdef USE_MUTEX_LOCK
-               giveSwallowMutex(&mutex);
-#endif
+               terminate(&mutex, -1, dpy);
 	       exit(EX_UNAVAILABLE);
           }
+          sig_globals.childPid = pid;
 
 	  D("Waiting for pid=%d\n", pid);
 
-	  handle_app(dpy, pid, &mutex);
+	  handle_app(dpy, &mutex);
 
 	  D("Wait done (repeats=%d, loops=%d)\n", repeats, loops);
 	  if (repeats < INF_LOOPS)
           {
 	       repeats -= loops;
 	  }
-    }
-
-    if(dpy)
-    {
-          XCloseDisplay(dpy);
-          sig_globals.dpy = 0;
      }
 
-     exit(0);
+     D("All done\n");
+     terminate(&mutex, -1, dpy);
+     return EXIT_SUCCESS;
 }
