@@ -404,10 +404,9 @@ static const char * get_home_dir(void)
  */
 static int get_cfg_path_prefix(const char * magic, char * buf, int bufLen)
 {
-     const char * fmt = NULL;
+     const char * fmt;
      int prefixLen = 1;
-     const char * home = get_home_dir();
-     const char * alt = getenv("MOZPLUGGER_HOME");
+     const char * home;
 
      if(is_base_mozplugger(magic))
      {
@@ -418,20 +417,29 @@ static int get_cfg_path_prefix(const char * magic, char * buf, int bufLen)
           prefixLen = strchr(magic, ':') - magic;
      }
 
-     if(alt)
+     /* Locations are ...
+      * $MOZPLUGGER_HOME/.cache/
+      * $XDG_CACHE_HOME/mozplugger/
+      * $HOME/.cache/mozplugger/
+      */
+
+     if( (home = getenv("MOZPLUGGER_HOME")) != NULL)
      {
-          fmt = "%s/.processed/%.*s";
-          home = alt;
+          fmt = "%s/.cache/%.*s";
+     }
+     else if( (home = getenv("XDG_CACHE_HOME")) != NULL)
+     {
+          fmt = "%s/mozplugger/%.*s";
+     }
+     else if( (home = get_home_dir()) != NULL)
+     {
+          fmt = "%s/.cache/mozplugger/%.*s";
      }
      else
      {
-          fmt = "%s/.mozplugger/.processed/%.*s";
-          if(!home)
-          {
-               reportError(NULL, "Mozplugger cannot determine HOME directory\n");
-               *buf = '\0';
-               return 0;
-          }
+          reportError(NULL, "Mozplugger cannot determine HOME directory\n");
+          *buf = '\0';
+          return 0;
      }
      return snprintf(buf, bufLen, fmt, home, prefixLen, magic);
 }
@@ -1578,7 +1586,7 @@ const char * NP2_GetMIMEDescription(const char * magic)
 
      if(!desc && update && !doesnt_exist && !haveError())
      {
-          reportError(NULL, "Please run mozplugger-update");
+          reportError(NULL, "Please close browser and run mozplugger-update");
      }
 
      if(haveError())
@@ -1646,17 +1654,36 @@ static const char * getPluginDescription(const char * magic)
      {
           snprintf(desc_buffer, sizeof(desc_buffer),
 		   "MozPlugger version " VERSION
-                   " Please run mozplugger-update, "
+                   " Refresh required, please close browser and run mozplugger-update, "
 		   "for documentation on mozplugger see the man page."
                   );
      }
      else
      {
+          const char * home = get_home_dir();
+          char * pCfg = NULL;
           struct stat details;
+          int i;
+
           details.st_mtime = 0;
           stat(config_fname, &details);
 
-          config_fname[strlen(config_fname)-4] = '\0';
+          /* removed cmds and replace with '*' */
+          i = strlen(config_fname)-4;
+          config_fname[i++] = '*';
+          config_fname[i] = '\0';
+
+          /* Hide the user's name!? */
+          i = strlen(home);
+          if(strncmp(home, config_fname, i) == 0)
+          {
+               pCfg = &config_fname[i-1];
+               *pCfg = '~';
+          }
+          else
+          {
+               pCfg = config_fname;
+          }
 
           snprintf(desc_buffer, sizeof(desc_buffer),
 		   "MozPlugger version "
@@ -1665,17 +1692,15 @@ static const char * getPluginDescription(const char * magic)
                    "(gcov)"
 #endif
 		   ", for documentation on mozplugger see the man page. "
-                   "Current cached config created on %s."
 		   "<table>"
-		   "<tr><td>Configuration files:</td><td>%s*</td></tr>"
+		   "<tr><td>Cached config files:</td><td>%s</td><td>%s</td></tr>"
                    "%s%s%s"
 		   " </table>"
 		   "<br clear=all>",
-                   asctime(localtime(&details.st_mtime)),
-		   config_fname ? config_fname : "Not found!",
+                   pCfg, asctime(localtime(&details.st_mtime)),
                    dbgPath ? "<tr><td>Debug file:</td><td>" : "",
                    dbgPath ? dbgPath : "",
-                   dbgPath ? "/" DEBUG_FILENAME "</td></tr>" : ""
+                   dbgPath ? "/" DEBUG_FILENAME "</td><td></td></tr>" : ""
                   );
      }
      free(config_fname);
