@@ -20,20 +20,11 @@
 #ifndef _MOZPLUGGER_H_
 #define _MOZPLUGGER_H_
 
-
 #define MAX_STATIC_MEMORY_POOL 65536
 
 /* Maximum size of the buffer used for environment variables. */
 #define ENV_BUFFER_SIZE 16348
 
-#define LARGE_BUFFER_SIZE 16384
-#define LARGE_BUFFER_SIZE_STR "16384"
-
-#define SMALL_BUFFER_SIZE 128
-#define SMALL_BUFFER_SIZE_STR "128"
-#define FIND_CACHE_SIZE 10
-
-
 extern bool is_playing(NPP instance);
 
 #endif
